#!/bin/bash

run_ce_solver()
{
	PROGRAMM_NAME="$1"
	LOG=$2
	maxSec=$3
	maxSecPerInstance=$4
	
	
	FILES=$(ls graphs/10thDimacs/*.* graphs/stanford/*.* graphs/webdata/*.*)
# 	FILES=$(ls graphs/*.dimacs)
	
	rm -f time.txt

	overallTime=$(date +%s);
	now=$(date +%s);
	elapsed=`expr $now - $overallTime`;
	notSolved=0
	showedHint=0

	for f in $FILES
	do
		if [ $elapsed -le $maxSec ]; then
			echo $f >> $LOG
			outF=${f//\//_}
			
			# start everything in a new process group such that we can kill everything if necessary
			(setsid /usr/bin/time -a -o time.txt $PROGRAMM_NAME $f -a3 -out $outF 1> prog_out.txt 2>&1) & PID=$!

			# kill processes on exit
			trap "{ kill -$PID 2>/dev/null; }" TERM
			trap "{ kill -9 -$PID 2>/dev/null; }" EXIT

			waited=0
			alive=0
			disown $PID;
			kill -0 $PID 2>/dev/null && alive=1;
			while [ $alive -eq 1 -a $waited -le $maxSecPerInstance ]; do
				sleep 2;
				(( waited += 2 ));
				kill -0 $PID 2>/dev/null || alive=0
			done
			if [ $alive -eq 1 ]; then
				# process still exists, kill it softly, then brutally, if necessary
				kill -TERM -$PID 2>/dev/null; sleep 1; kill -9 -$PID 2>/dev/null;
				finished=0;
			else
				finished=1;
			fi
		
			cat prog_out.txt >> $LOG

			rm -f prog_out.txt
		
			time=$(cat time.txt | awk '/system/{print $3}' | sed -e 's/elapsed//');

			echo -e $f "\ttime:" $time "s\tfinished:" $finished
			echo "" >> $LOG
			
			rm -f time.txt

			now=$(date +%s);
			elapsed=`expr $now - $overallTime`;
		else
			if [ $showedHint -eq 0 ]; then
				echo "maximal time of $maxSec sec elapsed. Script aborted."
				showedHint=1;
			fi		
		fi
	done
}


PROGRAMM_NAME="java RobustTwoClub"		# insert your program here
LOG=log.txt					# specify the name of the log file
maxSec=604800					# overall allowed time for the whole script
maxSecPerInstance=21600				# allowed time for one instance

echo "run $PROGRAMM_NAME"
run_ce_solver "$PROGRAMM_NAME" $LOG $maxSec $maxSecPerInstance

