#!/usr/bin/env python

from __future__ import print_function
from subprocess import TimeoutExpired, call
import shlex

import sys
import os
import argparse
import glob
import random
import subprocess as sp
import multiprocessing as mp

parser = argparse.ArgumentParser(description='A script for running triangle-2-club instances parallel on the server.')
parser.add_argument('version', type=int, help='ttclub version')
parser.add_argument('graphs', type=str, help='list of commands for the jar')
parser.add_argument('time_limit', type=int, help='time limit in seconds for each instance')
parser.add_argument('output_file', type=str, help='the file to write output too')
parser.add_argument('-var', '--variants', type=str, help='e = edge, v = vertex, ve = both')
parser.add_argument('-pc', '--processcount', type=int, help='the number of processes to run parallel')
parser.add_argument('-og', '--outputgraph', help='set this flag to add the vertices of the result to the outputs', action='store_true')
args = parser.parse_args()

#solver = '/export/storage/bachelor_philipp/experiments/ttclub.jar'
#solver = '/export/storage/implementation-triangle-clubs/ttclub/src/TwoClubs.class'
solver = 'TwoClubs.class'
ell = [1, 2, 3, 4, 5, 6, 7, 9, 11, 13, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85, 90, 100]

def work(command):
    """Defines the work unit on an input file"""
    #sp.call(command, shell=True)
    arguments = shlex.split(command)
    process = sp.Popen(arguments, shell=False)
    try:
        tl = args.time_limit * 2
        process.wait(timeout=tl)
    except TimeoutExpired:
        process.kill()
        print('Timeout: {}'.format(command))
    return 0

def get_file_info(line):
    infos = line.split(' ')
    if infos[0].startswith('"'):
        # combine [0] and [1]
        new_infos = []
        new_infos.append(infos[0] + ' ' + infos[1])
        for i in infos[2:]:
            new_infos.append(i)
        return new_infos
    return infos
        
def add_all_jobs(job_list, path, graph_type):
    edge = True
    vertex = True
    if args.variants:
        edge = 'e' in args.variants
        vertex = 'v' in args.variants

    #job = 'java -jar {} t {} {}'.format(solver, path, graph_type)
    job = 'java  {} t {} {}'.format(solver, path, graph_type)
    job += ' -t {} -s -o {} -SL'.format(args.time_limit, args.output_file)
    job = add_version_parameters(job, args.version)
    if args.outputgraph:
        job += ' -og'
    for l in ell:
        if vertex:
            job_list.append(job + ' -l ' + str(l))
        if edge:
            job_list.append(job + ' -e -l ' + str(l))

def add_single_job(job_list, info):
    job = 'java -jar {} t {} {}'.format(solver, info[0], info[1])
    job += ' -t {} -s -o {} -SL'.format(args.time_limit, args.output_file)
    edge_variant = False
    l_value = 1
    version = args.version
    for part in info:
        if part.startswith('e='):
            edge_variant = part[2:] == 'True'
        if part.startswith('l='):
            l_value = int(part[2:])
        if part.startswith('v='):
            version = int(part[2:])
    job += ' -l {}'.format(l_value)
    if edge_variant:
        job += ' -e'
    if args.outputgraph:
        job += ' -og'
    job_list.append(add_version_parameters(job, version))

def add_version_parameters(s, version):
    if version == 4:
        s += ' -lb 2'
    elif version == 2:
        s += ' -lb 0'
    elif version == 1:
        s += ' -lb 0 -cg 0'
    return s

def create_jobs(files):
    jobs = []
    for file in files:
        file_info = get_file_info(file)
        if len(file_info) < 2:
            continue
        elif len(file_info) == 2:
            add_all_jobs(jobs, file_info[0], file_info[1])
        elif file_info[2] == 'singlerun':
            add_single_job(jobs, file_info)
    return jobs
 
if __name__ == '__main__':
    #if args.solver:
    #    solver = args.solver
        
    files = []
    # list of files
    for line in open(args.graphs):
        if not line.startswith("#") and not line == '':
            files += [line.strip()]

    jobs = create_jobs(files)
    random.shuffle(jobs)
    
    #Set up the parallel task pool to use all available processors
    count = 12
    if args.processcount:
        count = max(1, min(24, args.processcount))
    pool = mp.Pool(processes=count)

    print('Starting {} jobs on {} processes.'.format(len(jobs), count))

    #Run the jobs	
    pool.map(work, jobs)
    
